searchData={"items":[{"type":"module","title":"Dataloader","doc":"# Dataloader\n\nDataloader provides an easy way efficiently load data in batches. It's\ninspired by [https://github.com/facebook/dataloader](https://github.com/facebook/dataloader), although it makes some\nsmall API changes to better suit Elixir use cases.\n\nCentral to Dataloader is the idea of a source. A single Dataloader struct can\nhave many different sources, which represent different ways to load data.\n\nHere's an example of a data loader using an ecto source, and then loading some\norganization data.\n\n```elixir\nsource = Dataloader.Ecto.new(MyApp.Repo)\n\n# setup the loader\nloader = Dataloader.new |> Dataloader.add_source(:db, source)\n\n# load some things\nloader =\n  loader\n  |> Dataloader.load(:db, Organization, 1)\n  |> Dataloader.load_many(:db, Organization, [4, 9])\n\n# actually retrieve them\nloader = Dataloader.run(loader)\n\n# Now we can get whatever values out we want\norganizations = Dataloader.get_many(loader, :db, Organization, [1,4])\n```\n\nThis will do a single SQL query to get all organizations by ids 1,4, and 9.\nYou can load multiple batches from multiple sources, and then when `run/1` is\ncalled batch will be loaded concurrently.\n\nHere we named the source `:db` within our dataloader. More commonly though if\nyou're using Phoenix you'll want to name it after one of your contexts, and\nhave a different source used for each context. This provides an easy way to\nenforce data access rules within each context. See the `Dataloader.Ecto`\nmoduledocs for more details","ref":"Dataloader.html"},{"type":"module","title":"Options - Dataloader","doc":"There are two configuration options:\n\n* `timeout` - The maximum timeout to wait for running a source, defaults to\n  1s more than the maximum timeout of all added sources. Set with care,\n  timeouts should really only be set on sources.\n* `get_policy` - This configures how the dataloader will behave when fetching\n  data which may have errored when we tried  to `load` it.\n\nThese can be set as part of the `new/1` call. So, for example, to\nconfigure a dataloader that returns `nil` on error with a 5s timeout:\n\n```elixir\nloader =\n  Dataloader.new(\n    get_policy: :return_nil_on_error,\n    timeout: :timer.seconds(5)\n  )\n```\n\n### `get_policy`\n\nThere are three implemented behaviours for this:\n\n* `raise_on_error` (default)- If successful, calling `get/4` or `get_many/4`\n  will return the value. If there was an exception when trying to load any of\n  the data, it will raise that exception\n* `return_nil_on_error` - Behaves similar to `raise_on_error` but will just\n  return `nil` instead of `raising`. It will still log errors\n* `tuples` - This will return `{:ok, value}`/`{:error, reason}` tuples\n  depending on a successful or failed load, allowing for more fine-grained\n  error handling if required","ref":"Dataloader.html#module-options"},{"type":"function","title":"Dataloader.add_source/3","doc":"","ref":"Dataloader.html#add_source/3"},{"type":"function","title":"Dataloader.async/1","doc":"","ref":"Dataloader.html#async/1"},{"type":"function","title":"Dataloader.async_safely/4","doc":"This is a helper method to run a set of async tasks in a separate supervision\ntree which:\n\n1. Is run by a supervisor linked to the main process. This ensures any async\n   tasks will get killed if the main process is killed.\n2. Spawns a separate task which traps exits for running the provided\n   function. This ensures we will always have some output, but are not\n   setting `:trap_exit` on the main process.\n\n**NOTE**: The provided `fun` must accept a `Task.Supervisor` as its first\nargument, as this function will prepend the relevant supervisor to `args`\n\nSee `run_task/3` for an example of a `fun` implementation, this will return\nwhatever that returns.","ref":"Dataloader.html#async_safely/4"},{"type":"function","title":"Dataloader.async_stream/3","doc":"","ref":"Dataloader.html#async_stream/3"},{"type":"function","title":"Dataloader.default_timeout/0","doc":"","ref":"Dataloader.html#default_timeout/0"},{"type":"function","title":"Dataloader.get/4","doc":"","ref":"Dataloader.html#get/4"},{"type":"function","title":"Dataloader.get_many/4","doc":"","ref":"Dataloader.html#get_many/4"},{"type":"function","title":"Dataloader.load/4","doc":"","ref":"Dataloader.html#load/4"},{"type":"function","title":"Dataloader.load_many/4","doc":"","ref":"Dataloader.html#load_many/4"},{"type":"function","title":"Dataloader.new/1","doc":"Create a new Dataloader instance.\n\nSee moduledoc for available options","ref":"Dataloader.html#new/1"},{"type":"function","title":"Dataloader.pending_batches?/1","doc":"","ref":"Dataloader.html#pending_batches?/1"},{"type":"function","title":"Dataloader.pmap/3","doc":"This used to be used by both the `Dataloader` module for running multiple\nsource queries concurrently, and the `KV` and `Ecto` sources to actually run\nseparate batch fetches (e.g. for `Posts` and `Users` at the same time).\n\nThe problem was that the behaviour between the sources and the parent\n`Dataloader` was actually slightly different. The `Dataloader`-specific\nbehaviour has been pulled out into `run_tasks/4`\n\nPlease use `async_safely/3` instead of this for fetching data from sources","ref":"Dataloader.html#pmap/3"},{"type":"function","title":"Dataloader.put/5","doc":"","ref":"Dataloader.html#put/5"},{"type":"function","title":"Dataloader.run/1","doc":"","ref":"Dataloader.html#run/1"},{"type":"function","title":"Dataloader.run_tasks/3","doc":"This helper function will call `fun` on all `items` asynchronously, returning\na map of `:ok`/`:error` tuples, keyed off the `items`. For example:\n\n    iex> Dataloader.run_tasks([1,2,3], fn x -> x * x end, [])\n    %{\n      1 => {:ok, 1},\n      2 => {:ok, 4},\n      3 => {:ok, 9}\n    }\n\nSimilarly, for errors:\n\n    iex> Dataloader.run_tasks([1,2,3], fn _x -> Process.sleep(5) end, [timeout: 1])\n    %{\n      1 => {:error, :timeout},\n      2 => {:error, :timeout},\n      3 => {:error, :timeout}\n    }\n\nBy default, tasks are run asynchronously. To run them synchronously, provide `async?: false`.","ref":"Dataloader.html#run_tasks/3"},{"type":"type","title":"Dataloader.option/0","doc":"","ref":"Dataloader.html#t:option/0"},{"type":"type","title":"Dataloader.source_name/0","doc":"","ref":"Dataloader.html#t:source_name/0"},{"type":"type","title":"Dataloader.t/0","doc":"","ref":"Dataloader.html#t:t/0"},{"type":"module","title":"Dataloader.Ecto","doc":"Ecto source for Dataloader\n\nThis defines a schema and an implementation of the `Dataloader.Source` protocol\nfor handling Ecto related batching.\n\nA simple Ecto source only needs to know about your application's Repo.","ref":"Dataloader.Ecto.html"},{"type":"module","title":"Basic Usage - Dataloader.Ecto","doc":"Querying by primary key (analogous to Ecto.Repo.get/3):\n\n```elixir\nsource = Dataloader.Ecto.new(MyApp.Repo)\n\nloader =\n  Dataloader.new\n  |> Dataloader.add_source(Accounts, source)\n  |> Dataloader.load(Accounts, User, 1)\n  |> Dataloader.load_many(Accounts, Organization, [4, 9])\n  |> Dataloader.run\n\norganizations = Dataloader.get_many(loader, Accounts, Organization, [4,9])\n```\n\nQuerying for associations. Here we look up the `:users` association on all\nthe organizations, and the `:organization` for a single user.\n\n```elixir\nloader =\n  loader\n  |> Dataloader.load(Accounts, :organization, user)\n  |> Dataloader.load_many(Accounts, :users, organizations)\n  |> Dataloader.run\n```\n\nQuerying by a column other than the primary key:\n\n```elixir\nloader =\n  loader\n  |> Dataloader.load(Accounts, {:one, User}, name: \"admin\")\n  |> Dataloader.run\n```\n\nHere we pass a keyword list of length one. It is only possible to\nquery by one column here; for more complex queries, see \"filtering\" below.\n\nNotice here that we need to also specify the cardinality in the batch_key\n(`:many` or `:one`), which will decide whether to return a list or a single\nvalue (or nil). This is because the column may not be a key and there may be\nmultiple matching records. Note also that even if we are returning `:many` values\nhere  from multiple matching records, this is still a call to `Dataloader.load/4`\nrather than `Dataloader.load_many/4` because there is only one val specified.","ref":"Dataloader.Ecto.html#module-basic-usage"},{"type":"module","title":"Filtering / Ordering - Dataloader.Ecto","doc":"`Dataloader.Ecto.new/2` can receive a 2 arity function that can be used to apply\nbroad ordering and filtering rules, as well as handle parameters\n\n```elixir\nsource = Dataloader.Ecto.new(MyApp.Repo, query: &Accounts.query/2)\n\nloader =\n  Dataloader.new\n  |> Dataloader.add_source(Accounts, source)\n```\n\nWhen we call `Dataloader.load/4` we can pass in a tuple as the batch key with a keyword list\nof parameters in addition to the queryable or assoc_field\n\n```elixir\n# with a queryable\nloader\n|> Dataloader.load(Accounts, {User, order: :name}, 1)\n\n# or an association\nloader\n|> Dataloader.load_many(Accounts, {:users, order: :name}, organizations)\n\n# this is still supported\nloader\n|> Dataloader.load(Accounts, User, 1)\n\n# as is this\nloader\n|> Dataloader.load(:accounts, :user, organization)\n```\n\nIn all cases the `Accounts.query` function would be:\n```elixir\ndef query(User, params) do\n  field = params[:order] || :id\n  from u in User, order_by: [asc: field(u, ^field)]\nend\ndef query(queryable, _) do\n  queryable\nend\n```\n\nIf we query something that ends up using the `User` schema, whether directly\nor via association, the `query/2` function will match on the first clause and\nwe can handle the params. If no params are supplied, the params arg defaults\nto `source.default_params` which itself defaults to `%{}`.\n\n`default_params` is an extremely useful place to store values like the current user:\n\n```elixir\nsource = Dataloader.Ecto.new(MyApp.Repo, [\n  query: &Accounts.query/2,\n  default_params: %{current_user: current_user},\n])\n\nloader =\n  Dataloader.new\n  |> Dataloader.add_source(Accounts, source)\n  |> Dataloader.load_many(Accounts, Organization, ids)\n  |> Dataloader.run\n\n# the query function\ndef query(Organization, %{current_user: user}) do\n  from o in Organization,\n    join: m in assoc(o, :memberships),\n    where: m.user_id == ^user.id\nend\ndef query(queryable, _) do\n  queryable\nend\n```\n\nIn our query function we are pattern matching on the current user to make sure\nthat we are only able to lookup data in organizations that the user actually\nhas a membership in. Additional options you specify IE `{Organization, %{order: :asc}}`\nare merged into the default.","ref":"Dataloader.Ecto.html#module-filtering-ordering"},{"type":"module","title":"Custom batch queries - Dataloader.Ecto","doc":"There are cases where you want to run the batch function yourself. To do this\nwe can add a custom `run_batch/5` callback to our source.\n\nThe `run_batch/5` function is executed with the query returned from the `query/2`\nfunction.\n\nFor example, we want to get the post count for a set of users.\n\nFirst we add a custom `run_batch/5` function.\n\n```\ndef run_batch(_, query, :post_count, users, repo_opts) do\n  user_ids = Enum.map(users, & &1.id)\n  default_count = 0\n\n  result =\n    query\n    |> where([p], p.user_id in ^user_ids)\n    |> group_by([p], p.user_id)\n    |> select([p], {p.user_id, count(\"*\")})\n    |> Repo.all(repo_opts)\n    |> Map.new()\n\n  for %{id: id} <- users do\n    [Map.get(result, id, default_count)]\n  end\nend\n\n# Fallback to original run_batch\ndef run_batch(queryable, query, col, inputs, repo_opts) do\n  Dataloader.Ecto.run_batch(Repo, queryable, query, col, inputs, repo_opts)\nend\n```\nThis function is supplied with a list of users, does a query and will return\nthe post count for each of user. If the user id is not found in the resultset,\nbecause the user has no posts, we return a post count of 0.\n\nNow we need to call `run_batch/5` from dataloader. First we add a few posts\nto the database.\n\nAfter that, the custom `run_batch/5` function is provided to the Dataloader\nsource. Now, we can load the post count for several users. When the dataloader\nruns it will call the custom `run_batch/5` and we can retrieve the posts counts\nfor each individual user.\n\n```\n[user1, user2] = [%User{id: 1}, %User{id: 2}]\n\nrows = [\n  %{user_id: user1.id, title: \"foo\", published: true},\n  %{user_id: user1.id, title: \"baz\", published: false}\n]\n\n_ = Repo.insert_all(Post, rows)\n\nsource =\n  Dataloader.Ecto.new(\n    Repo,\n    query: &query/2,\n    run_batch: &run_batch/5\n  )\n\nloader =\n  Dataloader.new()\n  |> Dataloader.add_source(Posts, source)\n\nloader =\n  loader\n  |> Dataloader.load(Posts, {:one, Post}, post_count: user1)\n  |> Dataloader.load(Posts, {:one, Post}, post_count: user2)\n  |> Dataloader.run()\n\n# Returns 2\nDataloader.get(loader, Posts, {:one, Post}, post_count: user1)\n# Returns 0\nDataloader.get(loader, Posts, {:one, Post}, post_count: user2)\n\n```\n\nAdditional params for the `query/2` function can be passed to the load functions\nwith a 3-tuple.\n\nFor example, to limit the above example to only return published we can add a query\nfunction to filter the published posts:\n\n```\ndef query(Post, %{published: published}) do\n  from p in Post,\n  where: p.published == ^published\nend\n\ndef query(queryable, _) do\n  queryable\nend\n```\n\nAnd we can return the published posts with a 3-tuple on the loader:\n\n```\nloader =\nloader\n|> Dataloader.load(Posts, {:one, Post}, post_count: user1)\n|> Dataloader.load(Posts, {:one, Post, %{published: true}}, post_count: user1)\n|> Dataloader.run()\n\n# Returns 2\nDataloader.get(loader, Posts, {:one, Post}, post_count: user1)\n# Returns 1\nDataloader.get(loader, Posts, {:one, Post, %{published: true}}, post_count: user1)\n```","ref":"Dataloader.Ecto.html#module-custom-batch-queries"},{"type":"function","title":"Dataloader.Ecto.new/2","doc":"Create an Ecto Dataloader source.\n\nThis module handles retrieving data from Ecto for dataloader. It requires a\nvalid Ecto Repo. It also accepts a `repo_opts:` option which is handy for\napplying options to any calls to Repo functions that this module makes.\n\nFor example, you can use this module in a multi-tenant context by using\nthe `prefix` option:\n\n```\nDataloader.Ecto.new(MyApp.Repo, repo_opts: [prefix: \"tenant\"])\n```","ref":"Dataloader.Ecto.html#new/2"},{"type":"function","title":"Dataloader.Ecto.run_batch/6","doc":"Default implementation for loading a batch. Handles looking up records by\ncolumn","ref":"Dataloader.Ecto.html#run_batch/6"},{"type":"type","title":"Dataloader.Ecto.batch_fun/0","doc":"","ref":"Dataloader.Ecto.html#t:batch_fun/0"},{"type":"type","title":"Dataloader.Ecto.opt/0","doc":"","ref":"Dataloader.Ecto.html#t:opt/0"},{"type":"type","title":"Dataloader.Ecto.query_fun/0","doc":"","ref":"Dataloader.Ecto.html#t:query_fun/0"},{"type":"type","title":"Dataloader.Ecto.repo_opts/0","doc":"","ref":"Dataloader.Ecto.html#t:repo_opts/0"},{"type":"type","title":"Dataloader.Ecto.t/0","doc":"","ref":"Dataloader.Ecto.html#t:t/0"},{"type":"module","title":"Dataloader.KV","doc":"Simple KV based Dataloader source.\n\nThis module is a simple key value based data loader source. You\nmust supply a function that accepts a batch key and list of ids,\nand returns a map of values keyed by id.\n\nValues may optionally be returned as :ok / :error tuples to indicate success of the operation.","ref":"Dataloader.KV.html"},{"type":"module","title":"Example - Dataloader.KV","doc":"```elixir\ndef datasource do\n  Dataloader.KV.new(&query/2, max_concurrency: 1)\nend\n\ndef query(:comments, posts) do\n  Map.new(posts, fn %{id: post_id} = post ->\n    {post, Comments.find_by(post_id: post_id)}\n  end)\nend\n```","ref":"Dataloader.KV.html#module-example"},{"type":"function","title":"Dataloader.KV.new/2","doc":"Create a KV Dataloader source.\n\nDataloader runs tasks concurrently using `Task.async_stream/3`. The\nconcurrency of a KV Dataloader source and the time tasks are allowed to run\ncan be controlled via options (see the \"Options\" section below).","ref":"Dataloader.KV.html#new/2"},{"type":"function","title":"Options - Dataloader.KV.new/2","doc":"* `:max_concurrency` - sets the maximum number of tasks to run at the same\n    time. Defaults to twice the number of schedulers online (see\n    `System.schedulers_online/0`).\n  * `:timeout` - the maximum amount of time (in milliseconds) a task is\n    allowed to execute for. Defaults to `30000`.\n  * `:async?` - set to `false` to disable the asynchronous behavior mentioned above.","ref":"Dataloader.KV.html#new/2-options"},{"type":"protocol","title":"Dataloader.Source","doc":"","ref":"Dataloader.Source.html"},{"type":"function","title":"Dataloader.Source.async?/1","doc":"Returns whether or not the source should be running synchronously or asynchronously with respect to the process making `Dataloader` function calls.","ref":"Dataloader.Source.html#async?/1"},{"type":"function","title":"Dataloader.Source.fetch/3","doc":"Fetch the result found under the given batch and item keys.","ref":"Dataloader.Source.html#fetch/3"},{"type":"function","title":"Dataloader.Source.load/3","doc":"Enqueue an item to be loaded under a given batch","ref":"Dataloader.Source.html#load/3"},{"type":"function","title":"Dataloader.Source.pending_batches?/1","doc":"Determine if there are any batches that have not yet been run.","ref":"Dataloader.Source.html#pending_batches?/1"},{"type":"function","title":"Dataloader.Source.put/4","doc":"Put a value into the results.\n\nUseful for warming caches. The source is permitted to reject the value.","ref":"Dataloader.Source.html#put/4"},{"type":"function","title":"Dataloader.Source.run/1","doc":"Run any batches queued up for this source.","ref":"Dataloader.Source.html#run/1"},{"type":"function","title":"Dataloader.Source.timeout/1","doc":"Returns the timeout (in ms) for the source.\n\nThis is important for ensuring the dataloader obeys the timeouts when running\nmultiple sources concurrently","ref":"Dataloader.Source.html#timeout/1"},{"type":"type","title":"Dataloader.Source.batch_key/0","doc":"","ref":"Dataloader.Source.html#t:batch_key/0"},{"type":"type","title":"Dataloader.Source.item_key/0","doc":"","ref":"Dataloader.Source.html#t:item_key/0"},{"type":"type","title":"Dataloader.Source.t/0","doc":"All the types that implement this protocol.","ref":"Dataloader.Source.html#t:t/0"},{"type":"exception","title":"Dataloader.GetError","doc":"","ref":"Dataloader.GetError.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v2.0.2 2024-12-19 - Changelog","doc":"- Bug Fix: correctly avoid double distinct ([#174](https://github.com/absinthe-graphql/dataloader/pull/174))","ref":"changelog.html#v2-0-2-2024-12-19"},{"type":"extras","title":"v2.0.1 2024-09-04 - Changelog","doc":"- Bug Fix: Revert undesirable lateral ([#171](https://github.com/absinthe-graphql/dataloader/pull/171))\n- Improvement: Relax otel process propagator vsn ([#167](https://github.com/absinthe-graphql/dataloader/pull/167)","ref":"changelog.html#v2-0-1-2024-09-04"},{"type":"extras","title":"v2.0.0 2023-07-24 - Changelog","doc":"- Breaking Feature: Automatically handle sync vs async for the ecto dataloader source ([#146](https://github.com/absinthe-graphql/dataloader/pull/146)). Other dataloader source implementations need to add an `async?` function to comply with the protocol. NOTE: This only impacts you if  you have a custom dataloader source. If you use the built in Ecto or KV sources then there is nothing you need to do.\n- Improvement: KV source no longer double wraps results in `{:ok, {:ok, value}} tuples` when you return an OK tuple while using the tuple policy ([#145](https://github.com/absinthe-graphql/dataloader/pull/145))","ref":"changelog.html#v2-0-0-2023-07-24"},{"type":"extras","title":"v1 - Changelog","doc":"For v1 changes see https://github.com/absinthe-graphql/dataloader/blob/v1/CHANGELOG.md","ref":"changelog.html#v1"},{"type":"extras","title":"Dataloader","doc":"# Dataloader\n\n[![Build Status](https://github.com/absinthe-graphql/dataloader/workflows/CI/badge.svg)](https://github.com/absinthe-graphql/dataloader/actions?query=workflow%3ACI)\n[![Version](https://img.shields.io/hexpm/v/dataloader.svg)](https://hex.pm/packages/dataloader)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/dataloader/)\n[![Download](https://img.shields.io/hexpm/dt/dataloader.svg)](https://hex.pm/packages/dataloader)\n[![License](https://img.shields.io/badge/License-MIT-blue.svg)](https://opensource.org/licenses/MIT)\n[![Last Updated](https://img.shields.io/github/last-commit/absinthe-graphql/dataloader.svg)](https://github.com/absinthe-graphql/dataloader/commits/master)\n\nDataloader provides an easy way efficiently load data in batches. It's inspired\nby [https://github.com/facebook/dataloader](https://github.com/facebook/dataloader), although it makes some small API changes to better suit Elixir use cases.","ref":"readme.html"},{"type":"extras","title":"Installation - Dataloader","doc":"The package can be installed by adding [`:dataloader`](https://hex.pm/packages/dataloader) to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:dataloader, \"~> 2.0.0\"}\n  ]\nend\n```\n\nNote: Dataloader requires Elixir 1.10 or higher.","ref":"readme.html#installation"},{"type":"extras","title":"Upgrading - Dataloader","doc":"See [CHANGELOG](./CHANGELOG.md) for upgrade steps between versions.","ref":"readme.html#upgrading"},{"type":"extras","title":"Documentation - Dataloader","doc":"- [Dataloader hexdocs](https://hexdocs.pm/dataloader).\n- For the tutorial, guides, and general information about Absinthe-related\n  projects, see [http://absinthe-graphql.org](http://absinthe-graphql.org).","ref":"readme.html#documentation"},{"type":"extras","title":"Usage - Dataloader","doc":"Central to Dataloader is the idea of a source. A single Dataloader struct can\nhave many different sources, which represent different ways to load data.\n\nHere's an example of a data loader using an Ecto source, and then loading some\norganization data.\n\n```elixir\nsource = Dataloader.Ecto.new(MyApp.Repo)\n\n# setup the loader\nloader = Dataloader.new |> Dataloader.add_source(:db, source)\n\n# load some things\nloader =\n  loader\n  |> Dataloader.load(:db, Organization, 1)\n  |> Dataloader.load_many(:db, Organization, [4, 9])\n\n# actually retrieve them\nloader = Dataloader.run(loader)\n\n# Now we can get whatever values out we want\norganizations = Dataloader.get_many(loader, :db, Organization, [1,4])\n```\n\nThis will do a single SQL query to get all organizations by ids 1, 4, and 9. You\ncan load multiple batches from multiple sources, and then when `run/1` is called\nbatch will be loaded concurrently.\n\nHere we named the source `:db` within our dataloader. More commonly though if\nyou're using Phoenix you'll want to name it after one of your contexts, and have\na different source used for each context. This provides an easy way to enforce\ndata access rules within each context. See the `Dataloader.Ecto` moduledocs for\nmore details","ref":"readme.html#usage"},{"type":"extras","title":"Sources - Dataloader","doc":"Dataloader ships with two different built in sources. The first is the Ecto source for easily pulling out data with ecto. The other is a simple `KV` key value source. See each module for its respective documentation.\n\nAnything that implements the `Dataloader.Source` protocol can act as a source.","ref":"readme.html#sources"},{"type":"extras","title":"Community - Dataloader","doc":"The project is under constant improvement by a growing list of\ncontributors, and your feedback is important. Please join us in Slack\n(`#absinthe-graphql` under the Elixir Slack account) or the Elixir Forum\n(tagged `absinthe`).\n\nPlease remember that all interactions in our official spaces follow\nour [Code of Conduct](./CODE_OF_CONDUCT.md).","ref":"readme.html#community"},{"type":"extras","title":"Related Projects - Dataloader","doc":"See the [GitHub organization](https://github.com/absinthe-graphql).","ref":"readme.html#related-projects"},{"type":"extras","title":"Contributing - Dataloader","doc":"Please follow [contribution guide](./CONTRIBUTING.md).","ref":"readme.html#contributing"},{"type":"extras","title":"License - Dataloader","doc":"See [LICENSE.md](./LICENSE.md).","ref":"readme.html#license"},{"type":"extras","title":"Telemetry","doc":"# Telemetry\n\nDataloader uses `telemetry` to instrument its activity.\n\nCall `:telemetry.attach/4` or `:telemetry.attach_many/4` to attach your\nhandler function to any of the following event names:\n\n- `[:dataloader, :source, :run, :start]` when the dataloader processing starts\n- `[:dataloader, :source, :run, :stop]` when the dataloader processing finishes\n- `[:dataloader, :source, :batch, :run, :start]` when the dataloader starts processing a single batch\n- `[:dataloader, :source, :batch, :run, :stop]` when the dataloader finishes processing a single batch\n\nTelemetry handlers are called with `measurements` and `metadata`. For details on\nwhat is passed, checkout `Dataloader`.","ref":"telemetry.html"},{"type":"extras","title":"Interactive Telemetry - Telemetry","doc":"As an example, you could attach a handler in an `iex -S mix` shell. Paste in:\n\n```elixir\n:telemetry.attach_many(\n  :demo,\n  [\n    [:dataloader, :source, :run, :stop]\n  ],\n  fn event_name, measurements, metadata, _config ->\n    %{\n      event_name: event_name,\n      measurements: measurements,\n      metadata: metadata\n    }\n    |> IO.inspect()\n  end,\n  []\n)\n```\n\nAfter a query is executed, you'll see something like:\n\n```elixir\n%{\n  event_name: [:dataloader, :source, :run, :stop],\n  measurements: %{duration: 112151},\n  metadata: %{\n    dataloader: %Dataloader{\n      options: [get_policy: :raise_on_error],\n      sources: ...\n    },\n    id: -576460752303420441\n  }\n}\n```","ref":"telemetry.html#interactive-telemetry"},{"type":"extras","title":"Opentelemetry - Telemetry","doc":"When using Opentelemetry, one usually wants to correlate spans that are created\nin spawned tasks with the main trace. For example, you might have a trace started\nin a Phoenix endpoint, and then have spans around database access.\n\nOne can correlate manually by attaching the OTel context the task function:\n\n```elixir\nctx = OpenTelemetry.Ctx.get_current()\n\nTask.async(fn ->\n  OpenTelemetry.Ctx.attach(ctx)\n\n  # do stuff that might create spans\nend)\n```\n\nWhen using Dataloader, the tasks are spawned by the loader itself, so you can't\nattach the context manually.\n\nInstead, you can add the `:opentelemetry_process_propagator` package to your\ndependencies, which has suitable wrappers that will attach the context\nautomatically. If the package is installed, Dataloader will use it in place\nof the default `Task.async/1` and `Task.async_stream/3`.","ref":"telemetry.html#opentelemetry"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}